# Copyright 2010 New Relic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x08v1.proto\x12\x15\x63om.newrelic.trace.v1"7\n\tSpanBatch\x12*\n\x05spans\x18\x01 \x03(\x0b\x32\x1b.com.newrelic.trace.v1.Span"\x86\x04\n\x04Span\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x12?\n\nintrinsics\x18\x02 \x03(\x0b\x32+.com.newrelic.trace.v1.Span.IntrinsicsEntry\x12H\n\x0fuser_attributes\x18\x03 \x03(\x0b\x32/.com.newrelic.trace.v1.Span.UserAttributesEntry\x12J\n\x10\x61gent_attributes\x18\x04 \x03(\x0b\x32\x30.com.newrelic.trace.v1.Span.AgentAttributesEntry\x1aX\n\x0fIntrinsicsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.com.newrelic.trace.v1.AttributeValue:\x02\x38\x01\x1a\\\n\x13UserAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.com.newrelic.trace.v1.AttributeValue:\x02\x38\x01\x1a]\n\x14\x41gentAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.com.newrelic.trace.v1.AttributeValue:\x02\x38\x01"t\n\x0e\x41ttributeValue\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x14\n\nbool_value\x18\x02 \x01(\x08H\x00\x12\x13\n\tint_value\x18\x03 \x01(\x03H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x42\x07\n\x05value"%\n\x0cRecordStatus\x12\x15\n\rmessages_seen\x18\x01 \x01(\x04\x32\xc5\x01\n\rIngestService\x12T\n\nRecordSpan\x12\x1b.com.newrelic.trace.v1.Span\x1a#.com.newrelic.trace.v1.RecordStatus"\x00(\x01\x30\x01\x12^\n\x0fRecordSpanBatch\x12 .com.newrelic.trace.v1.SpanBatch\x1a#.com.newrelic.trace.v1.RecordStatus"\x00(\x01\x30\x01\x62\x06proto3'
)


_SPANBATCH = DESCRIPTOR.message_types_by_name["SpanBatch"]
_SPAN = DESCRIPTOR.message_types_by_name["Span"]
_SPAN_INTRINSICSENTRY = _SPAN.nested_types_by_name["IntrinsicsEntry"]
_SPAN_USERATTRIBUTESENTRY = _SPAN.nested_types_by_name["UserAttributesEntry"]
_SPAN_AGENTATTRIBUTESENTRY = _SPAN.nested_types_by_name["AgentAttributesEntry"]
_ATTRIBUTEVALUE = DESCRIPTOR.message_types_by_name["AttributeValue"]
_RECORDSTATUS = DESCRIPTOR.message_types_by_name["RecordStatus"]
SpanBatch = _reflection.GeneratedProtocolMessageType(
    "SpanBatch",
    (_message.Message,),
    {
        "DESCRIPTOR": _SPANBATCH,
        "__module__": "v1_pb2"
        # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.SpanBatch)
    },
)
_sym_db.RegisterMessage(SpanBatch)

Span = _reflection.GeneratedProtocolMessageType(
    "Span",
    (_message.Message,),
    {
        "IntrinsicsEntry": _reflection.GeneratedProtocolMessageType(
            "IntrinsicsEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _SPAN_INTRINSICSENTRY,
                "__module__": "v1_pb2"
                # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.Span.IntrinsicsEntry)
            },
        ),
        "UserAttributesEntry": _reflection.GeneratedProtocolMessageType(
            "UserAttributesEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _SPAN_USERATTRIBUTESENTRY,
                "__module__": "v1_pb2"
                # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.Span.UserAttributesEntry)
            },
        ),
        "AgentAttributesEntry": _reflection.GeneratedProtocolMessageType(
            "AgentAttributesEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _SPAN_AGENTATTRIBUTESENTRY,
                "__module__": "v1_pb2"
                # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.Span.AgentAttributesEntry)
            },
        ),
        "DESCRIPTOR": _SPAN,
        "__module__": "v1_pb2"
        # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.Span)
    },
)
_sym_db.RegisterMessage(Span)
_sym_db.RegisterMessage(Span.IntrinsicsEntry)
_sym_db.RegisterMessage(Span.UserAttributesEntry)
_sym_db.RegisterMessage(Span.AgentAttributesEntry)

AttributeValue = _reflection.GeneratedProtocolMessageType(
    "AttributeValue",
    (_message.Message,),
    {
        "DESCRIPTOR": _ATTRIBUTEVALUE,
        "__module__": "v1_pb2"
        # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.AttributeValue)
    },
)
_sym_db.RegisterMessage(AttributeValue)

RecordStatus = _reflection.GeneratedProtocolMessageType(
    "RecordStatus",
    (_message.Message,),
    {
        "DESCRIPTOR": _RECORDSTATUS,
        "__module__": "v1_pb2"
        # @@protoc_insertion_point(class_scope:com.newrelic.trace.v1.RecordStatus)
    },
)
_sym_db.RegisterMessage(RecordStatus)

_INGESTSERVICE = DESCRIPTOR.services_by_name["IngestService"]
if _descriptor._USE_C_DESCRIPTORS is False:
    DESCRIPTOR._options = None
    _SPAN_INTRINSICSENTRY._options = None
    _SPAN_INTRINSICSENTRY._serialized_options = b"8\001"
    _SPAN_USERATTRIBUTESENTRY._options = None
    _SPAN_USERATTRIBUTESENTRY._serialized_options = b"8\001"
    _SPAN_AGENTATTRIBUTESENTRY._options = None
    _SPAN_AGENTATTRIBUTESENTRY._serialized_options = b"8\001"
    _SPANBATCH._serialized_start = 35
    _SPANBATCH._serialized_end = 90
    _SPAN._serialized_start = 93
    _SPAN._serialized_end = 611
    _SPAN_INTRINSICSENTRY._serialized_start = 334
    _SPAN_INTRINSICSENTRY._serialized_end = 422
    _SPAN_USERATTRIBUTESENTRY._serialized_start = 424
    _SPAN_USERATTRIBUTESENTRY._serialized_end = 516
    _SPAN_AGENTATTRIBUTESENTRY._serialized_start = 518
    _SPAN_AGENTATTRIBUTESENTRY._serialized_end = 611
    _ATTRIBUTEVALUE._serialized_start = 613
    _ATTRIBUTEVALUE._serialized_end = 729
    _RECORDSTATUS._serialized_start = 731
    _RECORDSTATUS._serialized_end = 768
    _INGESTSERVICE._serialized_start = 771
    _INGESTSERVICE._serialized_end = 968
# @@protoc_insertion_point(module_scope)
