# Copyright 2010 New Relic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import memcache
from newrelic.hooks.datastore_memcache import (_memcache_client_methods,
        _memcache_multi_methods)

def test_all_methods_wrapped():
    client = memcache.Client([])

    all_methods = (list(_memcache_client_methods) +
            list(_memcache_multi_methods))

    for m in all_methods:
        method = getattr(client, m)
        assert hasattr(method, '__wrapped__')
