# Copyright 2010 New Relic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.exceptions import ObjectDoesNotExist
from tastypie.resources import Resource
from tastypie.exceptions import NotFound


class SimpleResource(Resource):
    class Meta:
        resource_name = 'simple'

    def obj_get(self, *args, **kwargs):
        pk = kwargs['pk']
        if pk == 'NotFound':
            raise NotFound('Object not found.')
        elif pk == 'ObjectDoesNotExist':
            raise ObjectDoesNotExist('Object does not exist.')
        elif pk == 'ZeroDivisionError':
            1 / 0
        else:
            raise NotImplemented()
